@@ -10,7 +10,7 @@
 //! ```
 //! use circular_queue::CircularQueue;
 //!
-//! let mut queue = CircularQueue::new(3);
+//! let mut queue = CircularQueue::with_capacity(3);
 //! queue.push(1);
 //! queue.push(2);
 //! queue.push(3);
@@ -32,6 +32,9 @@ use std::slice::{Iter as SliceIter, IterMut as SliceIterMut};
 #[derive(Clone, Debug)]
 pub struct CircularQueue<T> {
     data: Vec<T>,
+    // Using our own capacity instead of the one stored in Vec to ensure consistent behavior with
+    // zero-sized types.
+    capacity: usize,
     insertion_index: usize,
 }
 
@@ -42,7 +45,7 @@ pub type Iter<'a, T> = Chain<Rev<SliceIter<'a, T>>, Rev<SliceIter<'a, T>>>;
 pub type IterMut<'a, T> = Chain<Rev<SliceIterMut<'a, T>>, Rev<SliceIterMut<'a, T>>>;
 
 impl<T> CircularQueue<T> {
-    /// Constructs a new, empty `CircularQueue<T>`.
+    /// Constructs a new, empty `CircularQueue<T>` with the requested capacity.
     ///
     /// # Panics
     ///
@@ -53,16 +56,17 @@ impl<T> CircularQueue<T> {
     /// ```
     /// use circular_queue::CircularQueue;
     ///
-    /// let mut queue: CircularQueue<i32> = CircularQueue::new(5);
+    /// let mut queue: CircularQueue<i32> = CircularQueue::with_capacity(5);
     /// ```
     #[inline]
-    pub fn new(capacity: usize) -> Self {
+    pub fn with_capacity(capacity: usize) -> Self {
         if capacity == 0 {
             panic!("capacity must be greater than 0");
         }
 
         Self {
             data: Vec::with_capacity(capacity),
+            capacity,
             insertion_index: 0,
         }
     }
@@ -74,7 +78,7 @@ impl<T> CircularQueue<T> {
     /// ```
     /// use circular_queue::CircularQueue;
     ///
-    /// let mut queue = CircularQueue::new(5);
+    /// let mut queue = CircularQueue::with_capacity(5);
     /// queue.push(1);
     /// queue.push(2);
     /// queue.push(3);
@@ -86,6 +90,24 @@ impl<T> CircularQueue<T> {
         self.data.len()
     }
 
+    /// Returns `true` if the queue contains no elements.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use circular_queue::CircularQueue;
+    ///
+    /// let mut queue = CircularQueue::with_capacity(5);
+    /// assert!(queue.is_empty());
+    ///
+    /// queue.push(1);
+    /// assert!(!queue.is_empty());
+    /// ```
+    #[inline]
+    pub fn is_empty(&self) -> bool {
+        self.data.is_empty()
+    }
+
     /// Returns the capacity of the queue.
     ///
     /// # Examples
@@ -93,12 +115,12 @@ impl<T> CircularQueue<T> {
     /// ```
     /// use circular_queue::CircularQueue;
     ///
-    /// let queue: CircularQueue<i32> = CircularQueue::new(5);
+    /// let queue: CircularQueue<i32> = CircularQueue::with_capacity(5);
     /// assert_eq!(queue.capacity(), 5);
     /// ```
     #[inline]
     pub fn capacity(&self) -> usize {
-        self.data.capacity()
+        self.capacity
     }
 
     /// Clears the queue.
@@ -108,7 +130,7 @@ impl<T> CircularQueue<T> {
     /// ```
     /// use circular_queue::CircularQueue;
     ///
-    /// let mut queue = CircularQueue::new(5);
+    /// let mut queue = CircularQueue::with_capacity(5);
     /// queue.push(1);
     /// queue.push(2);
     /// queue.push(3);
@@ -131,7 +153,7 @@ impl<T> CircularQueue<T> {
     /// ```
     /// use circular_queue::CircularQueue;
     ///
-    /// let mut queue = CircularQueue::new(3);
+    /// let mut queue = CircularQueue::with_capacity(3);
     /// queue.push(1);
     /// queue.push(2);
     /// queue.push(3);
@@ -146,13 +168,13 @@ impl<T> CircularQueue<T> {
     /// assert_eq!(iter.next(), Some(&2));
     /// ```
     pub fn push(&mut self, x: T) {
-        if self.data.len() < self.data.capacity() {
+        if self.data.len() < self.capacity() {
             self.data.push(x);
         } else {
             self.data[self.insertion_index] = x;
         }
 
-        self.insertion_index = (self.insertion_index + 1) % self.data.capacity();
+        self.insertion_index = (self.insertion_index + 1) % self.capacity();
     }
 
     /// Returns an iterator over the queue's contents.
@@ -164,7 +186,7 @@ impl<T> CircularQueue<T> {
     /// ```
     /// use circular_queue::CircularQueue;
     ///
-    /// let mut queue = CircularQueue::new(3);
+    /// let mut queue = CircularQueue::with_capacity(3);
     /// queue.push(1);
     /// queue.push(2);
     /// queue.push(3);
@@ -177,7 +199,7 @@ impl<T> CircularQueue<T> {
     /// assert_eq!(iter.next(), Some(&2));
     /// ```
     #[inline]
-    pub fn iter<'a>(&'a self) -> Iter<'a, T> {
+    pub fn iter(&self) -> Iter<T> {
         let (a, b) = self.data.split_at(self.insertion_index);
         a.iter().rev().chain(b.iter().rev())
     }
@@ -191,7 +213,7 @@ impl<T> CircularQueue<T> {
     /// ```
     /// use circular_queue::CircularQueue;
     ///
-    /// let mut queue = CircularQueue::new(3);
+    /// let mut queue = CircularQueue::with_capacity(3);
     /// queue.push(1);
     /// queue.push(2);
     /// queue.push(3);
@@ -204,7 +226,7 @@ impl<T> CircularQueue<T> {
     /// assert_eq!(iter.next(), Some(&mut 2));
     /// ```
     #[inline]
-    pub fn iter_mut<'a>(&'a mut self) -> IterMut<'a, T> {
+    pub fn iter_mut(&mut self) -> IterMut<T> {
         let (a, b) = self.data.split_at_mut(self.insertion_index);
         a.iter_mut().rev().chain(b.iter_mut().rev())
     }
@@ -217,19 +239,19 @@ mod tests {
     #[test]
     #[should_panic]
     fn zero_capacity() {
-        let _ = CircularQueue::<i32>::new(0);
+        let _ = CircularQueue::<i32>::with_capacity(0);
     }
 
     #[test]
     fn empty_queue() {
-        let q = CircularQueue::<i32>::new(5);
+        let q = CircularQueue::<i32>::with_capacity(5);
 
         assert_eq!(q.iter().next(), None);
     }
 
     #[test]
     fn partially_full_queue() {
-        let mut q = CircularQueue::new(5);
+        let mut q = CircularQueue::with_capacity(5);
         q.push(1);
         q.push(2);
         q.push(3);
@@ -242,7 +264,7 @@ mod tests {
 
     #[test]
     fn full_queue() {
-        let mut q = CircularQueue::new(5);
+        let mut q = CircularQueue::with_capacity(5);
         q.push(1);
         q.push(2);
         q.push(3);
@@ -257,7 +279,7 @@ mod tests {
 
     #[test]
     fn over_full_queue() {
-        let mut q = CircularQueue::new(5);
+        let mut q = CircularQueue::with_capacity(5);
         q.push(1);
         q.push(2);
         q.push(3);
@@ -274,7 +296,7 @@ mod tests {
 
     #[test]
     fn clear() {
-        let mut q = CircularQueue::new(5);
+        let mut q = CircularQueue::with_capacity(5);
         q.push(1);
         q.push(2);
         q.push(3);
@@ -300,7 +322,7 @@ mod tests {
 
     #[test]
     fn mutable_iterator() {
-        let mut q = CircularQueue::new(5);
+        let mut q = CircularQueue::with_capacity(5);
         q.push(1);
         q.push(2);
         q.push(3);
@@ -316,4 +338,23 @@ mod tests {
         let res: Vec<_> = q.iter().map(|&x| x).collect();
         assert_eq!(res, [14, 12, 10, 8, 6]);
     }
+
+    #[test]
+    fn zero_sized() {
+        let mut q = CircularQueue::with_capacity(3);
+        assert_eq!(q.capacity(), 3);
+
+        q.push(());
+        q.push(());
+        q.push(());
+        q.push(());
+
+        assert_eq!(q.len(), 3);
+
+        let mut iter = q.iter();
+        assert_eq!(iter.next(), Some(&()));
+        assert_eq!(iter.next(), Some(&()));
+        assert_eq!(iter.next(), Some(&()));
+        assert_eq!(iter.next(), None);
+    }
 }
